#!/usr/bin/perl 

##
##	Help Info
##

my $info = <<INFO;
send_nscafe Perl Client v1.0 for NSCAFE 1.0
Last Modified 4/14/06 by Groundwork Open Source
Based on send_nsca.pl by Jason Lancaster

Usage: $0 -H <host_address> [-f file_to_send] [-p port] [-t timeout] [-d]

Options:
<host_address>\t= The IP address of the host running the NSCAFE daemon
[file_to_send]\t= Filename to read and send
\t\t  (use instead of STDIN)
[port]\t\t= The port on which the daemon is running - default is 5668
[timeout]\t= Number of seconds before connection attempt times out
\t\t  (default timeout is 10 seconds)
[d]\t\t= Debug mode
[h]\t\t= Print this help file

Note:
The only supported data encryption method is XOR.
This program can accept data through STDIN using tab delimited values.
 eg, cat stats.out | $0

INFO

##
##
##

use strict;
use Getopt::Std;
use Socket; 

##
## Main program's ugly global variables 
##

my (@crc32_table);    # we really don't need this to be global
my (%arg, $arg);
my $xml_message;

##
## Define options and Set option defaults
##

getopts("df:H:hp:t:k:", \%arg);
$arg{d} = 0 unless $arg{d};	# debug mode
$arg{f} = "" unless $arg{f};	# file to send
$arg{h} = 0 unless $arg{h};	# help
$arg{p} = 5668 unless $arg{p};	# port # of nscafe
$arg{t} = 5 unless $arg{t};	# timeout
$arg{k} = "SERVICE_ALERT" unless $arg{k};

##
## Do some basic option handling and set requirements
## Die if no -H defined, the bare minimum necessary to run.
##

if ($arg{d} == 1) {
	open (LOG, ">>/usr/local/groundwork/nscafe/send_nscafe.log");
}

if ($arg{h} == 1) { &helpit; }
unless ($arg{H}) { warn "Missing required option -H <host_address>\n\n"; &helpit; }

#######################
#     subroutines     # 
#######################

sub helpit
{
    print $info;
    exit 0;
}

##
## Read STDIN and parse it
##

sub process_service_checks
{
    my ($iaddr, $paddr, $proto, $xor_key, $timestamp);
    my ($d);
    my $packet_version = "3";
    my $failure = sprintf("Could not establish a connection to %s! Are you an allowed_host?\n\n", $arg{H});

    $iaddr   = inet_aton($arg{H});
    $paddr   = sockaddr_in($arg{p}, $iaddr);
    $proto   = getprotobyname('tcp');

    # Start the timer. We don't want stale processes!
    local $SIG{ALRM} = sub { die sprintf("Connection to %s timed out after %d seconds\n\n", $arg{H}, $arg{t}); };       
    alarm $arg{t};
    
    # Open the socket
    socket (SOCK, PF_INET, SOCK_STREAM, $proto) || die $failure;
    connect(SOCK, $paddr) || die $failure;

    # it used to read the timestamp. That is interesting, b/c
    # the timestamp used to be calculated from
    # that side, why on earth would we need it?!
    # read(SOCK,$timestamp, 4) || die $failure;

    if ($arg{f} ne "")
    {
        open(STDIN, "<$arg{f}") || die "Cannot open $arg{f} for reading, $!\n\n";
    }
    
    ##
    ## One line per service check to send.
    ##

    my $hostname;
    my $service;
    my $return_code;
    my $status;
    my $perfdata;

    while (my $line = <STDIN>)
    {
	my $tmp;

        chomp $line;
	$d = $d + 1;
	if ($arg{d} == 1) {
		print LOG "****** TYPE=$arg{k} *********\n";
	}

	if ($arg{k} eq "SERVICE_ALERT")
	{	# SERVICE_ALERT:
		print LOG "**** SERVICE_ALERT\n";
        	($hostname,$service,$return_code,$status,$perfdata) = split(/\t/, $line);

		if ($arg{d} == 1) {
	        	print LOG "Read SERVICE_ALERT input: $hostname\t$service\t$return_code\t$status\t$perfdata\n";
		}

		$xml_message = "<NSCAFE ";
	        $xml_message .= "Host=\"$hostname\" ";                                      # Default Identification - should set to IP address if known
        	$xml_message .= "ServiceDescription=\"$service\" ";                     # Invalid field??
	        $xml_message .= "Severity=\"$return_code\" ";
        	$xml_message .= "ServiceStatus=\"$return_code\" ";
		if ($perfdata ne "") {
        		$status .=  "|".$perfdata;                                              # Append perf data to output
		}
        	$tmp = $status;
        	$tmp =~ s/["']/ /g;
        	$xml_message .= "TextMessage=\"$tmp\" ";
        	$tmp = time;                    # Send UTC time
        	$xml_message .= "ReportDate=\"$tmp\" ";
        	$tmp = time;
        	$xml_message .= "LastInsertDate=\"$tmp\" ";
        	$xml_message .= "SubComponent=\"$hostname:$service\" ";
        	$xml_message .= "ErrorType=\"$arg{k}\" ";
		$xml_message .= "/>\n";
	}
	else
	{	# HOST_ALERT:
        	($hostname,$return_code,$status,$perfdata) = split(/\t/, $line);

		if ($arg{d} == 1) {
	        	print LOG "Read HOST_ALERT input: $hostname\t$return_code\t$status\t$perfdata\n";
		}
		$xml_message = "<NSCAFE ";
	        $xml_message .= "Host=\"$hostname\" ";                                      # Default Identification - should set to IP address if known
        	$xml_message .= "ServiceDescription=\"NONE\" ";                     # Invalid field??
	        $xml_message .= "Severity=\"$return_code\" ";
        	$xml_message .= "ServiceStatus=\"$return_code\" ";
		if ($perfdata ne "") {
        		$status .=  "|".$perfdata;                                              # Append perf data to output
		}
        	$tmp = $status;
        	$tmp =~ s/["']/ /g;
        	$xml_message .= "TextMessage=\"$tmp\" ";
        	$tmp = time;                    # Send UTC time
        	$xml_message .= "ReportDate=\"$tmp\" ";
        	$tmp = time;
        	$xml_message .= "LastInsertDate=\"$tmp\" ";
        	$xml_message .= "SubComponent=\"$hostname\" ";
        	$xml_message .= "ErrorType=\"$arg{k}\" ";
		$xml_message .= "/>\n";
	}

	if ($arg{d} == 1) {
		print LOG "xml_message=$xml_message\n";
	}

        send(SOCK,$xml_message,0) || warn sprintf("Could not send XML packet %d\n", $d);
    } 

    ##
    ## Send CLOSE XML Command.
    ##

    $xml_message = "<CLOSE>\n";
    send(SOCK,$xml_message,0) || warn sprintf("Could not send close XML command.\n");

    close(STATS);
    close(SOCK);

    # Things went good. Reset alarm.
    alarm 0;
    
    if ($arg{d} == 1) {
    	print LOG "Sent $d packets to $arg{H}\n";
	close(LOG);
    }

    exit 0;
}

##
## Main program
##

&process_service_checks();
exit;

##
